# ■ランダム Koch 曲線を利用した島形状の生成
#
#   曲線の X 座標を回転方向、Y 座標を半径方向に変換、
#   すなわち極座標に変換することで島形状を生成します。
#   ただし、最初の Y 座標と最後の Y 座標を合わせることで
#   最初と最後がつながるよう調整します。
#
#   赤色の線 　　　： 最初に求められた曲線
#   緑色の線 　　　： 調整後の曲線
#   薄黄色の多角形 ： 生成された島形状
#
#   マウスの左ボタンクリックで新しい島形状を生成します。
#
from tkinter import *
import math, random, copy
X0=350; Y0=350 # 島中央の座標
R0=50          # Y = 0のときの半径
curX=0; curY=0; Theta=0; PI=math.pi
def t_pos(X, Y): global curX, curY; curX = X; curY = Y
def t_deg(T)   : global Theta; Theta  = T
def t_turn(T)  : global Theta; Theta += T
def t_forward(length):
    global curX, curY, Theta, PI,canvas
    TH = Theta * PI / 180;
    X  = curX+length*math.cos(TH)
    Y  = curY+length*math.sin(TH)
    canvas.create_line(curX, 200 - curY, X, 200 - Y,
                       width=3, fill="#FF0000")
    curX=X; curY=Y
    hist.append(copy.copy(X));hist.append(copy.copy(Y));
def koch(n, length):
    if n<=0: t_forward(length); return
    #方向、長さともに乱数を発生させて進む
    R1=(random.random()+1)*0.2; R2=(random.random()+1)*0.2
    R3=1-(R1+R2)
    T1=60*random.random();T2=-120*random.random()
    T3=-(T1+T2)
    nn=n -1
    koch(nn,length*R1)
    t_turn(T1); koch(nn,length*R2)
    t_turn(T2); koch(nn,length*R2)
    t_turn(T3); koch(nn,length*R3)

def randomKochIland():
    global hist,canvas
    canvas.delete(ALL)
    
    hist=[0,0]
    t_pos(0,0); t_deg(0); koch(5, 200)
    LM=len(hist);NM=int(LM/2)
    DY=(hist[LM-1]-hist[1])/NM/2
    ii=3
    for i in range(NM-1): hist[ii]-=DY*(ii-2); ii+=2
    ii=0
    for i in range(NM-1):
        canvas.create_line(hist[ii], 200 - hist[ii+1],
                           hist[ii+2], 200 -hist[ii+3],
                           width=3, fill="#007700")
        ii+=2
    canvas.create_line(hist[0], 200 - hist[1],
                       hist[LM-2], 200 -hist[LM-1],
                       width=2, fill="#0000FF")
    PDT=[0 for i in range(LM)]
    ii=0;dth=2*PI/hist[LM-2]
    for i in range(NM):
        th=dth*hist[ii]
        PDT[ii  ]=X0+(R0+hist[ii+1])*math.cos(th)
        PDT[ii+1]=Y0+(R0+hist[ii+1])*math.sin(th)
        ii+=2
    canvas.create_polygon(PDT, fill="#FFFF77", outline="#FF7700", width=3)
    root.update()
def leftMouseDown(event):#ダウン
    randomKochIland()

root=Tk(); root.title("Random Koch and The Island")
canvas=Canvas(root, width=580, height=558,bg="#CCDDFF")
canvas.pack()

randomKochIland()
canvas.bind("<Button-1>",leftMouseDown)

root.mainloop()


